@@ -21,9 +21,7 @@ from __future__ import absolute_import
 from __future__ import print_function
 from __future__ import unicode_literals
 
-from copy import deepcopy
 import fnmatch
-import functools
 import hawkey
 import itertools
 import libdnf.transaction
@@ -37,7 +35,7 @@ import time
 from dnf.cli.format import format_number, format_time
 from dnf.i18n import _, C_, P_, ucd, fill_exact_width, textwrap_fill, exact_width, select_short_long
 from dnf.pycomp import xrange, basestring, long, unicode, sys_maxsize
-from dnf.yum.rpmtrans import LoggingTransactionDisplay
+from dnf.yum.rpmtrans import TransactionDisplay
 from dnf.db.history import MergedTransactionWrapper
 import dnf.base
 import dnf.callback
@@ -52,50 +50,6 @@ import dnf.yum.misc
 
 logger = logging.getLogger('dnf')
 
-def _make_lists(transaction, goal):
-    b = dnf.util.Bunch({
-        'downgraded': [],
-        'erased': [],
-        'erased_clean': [],
-        'erased_dep': [],
-        'installed': [],
-        'installed_group': [],
-        'installed_dep': [],
-        'installed_weak': [],
-        'reinstalled': [],
-        'upgraded': [],
-        'failed': [],
-    })
-
-    for tsi in transaction:
-        if tsi.state == libdnf.transaction.TransactionItemState_ERROR:
-            b.failed.append(tsi)
-        elif tsi.action == libdnf.transaction.TransactionItemAction_DOWNGRADE:
-            b.downgraded.append(tsi)
-        elif tsi.action == libdnf.transaction.TransactionItemAction_INSTALL:
-            if tsi.reason == libdnf.transaction.TransactionItemReason_GROUP:
-                b.installed_group.append(tsi)
-            elif tsi.reason == libdnf.transaction.TransactionItemReason_DEPENDENCY:
-                b.installed_dep.append(tsi)
-            elif tsi.reason == libdnf.transaction.TransactionItemReason_WEAK_DEPENDENCY:
-                b.installed_weak.append(tsi)
-            else:
-                # TransactionItemReason_USER
-                b.installed.append(tsi)
-        elif tsi.action == libdnf.transaction.TransactionItemAction_REINSTALL:
-            b.reinstalled.append(tsi)
-        elif tsi.action == libdnf.transaction.TransactionItemAction_REMOVE:
-            if tsi.reason == libdnf.transaction.TransactionItemReason_CLEAN:
-                b.erased_clean.append(tsi)
-            elif tsi.reason == libdnf.transaction.TransactionItemReason_DEPENDENCY:
-                b.erased_dep.append(tsi)
-            else:
-                b.erased.append(tsi)
-        elif tsi.action == libdnf.transaction.TransactionItemAction_UPGRADE:
-            b.upgraded.append(tsi)
-
-    return b
-
 
 def _spread_in_columns(cols_count, label, lst):
     left = itertools.chain((label,), itertools.repeat(''))
@@ -119,7 +73,7 @@ class Output(object):
     def __init__(self, base, conf):
         self.conf = conf
         self.base = base
-        self.term = dnf.cli.term.Term()
+        self.term = dnf.cli.term.Term(color=base.conf.color)
         self.progress = None
 
     def _banner(self, col_data, row):
@@ -251,7 +205,7 @@ class Output(object):
                     full_columns.append(col[-1][0])
                 else:
                     full_columns.append(columns[d] + 1)
-            full_columns[0] += len(indent)
+            full_columns[0] += len(indent) * 2
             # if possible, try to keep default width (usually 80 columns)
             default_width = self.term.columns
             if sum(full_columns) > default_width:
@@ -643,18 +597,10 @@ class Output(object):
                        number
                  '>' - highlighting used when the package has a higher version
                        number
-        :return: (exit_code, [errors])
-
-        exit_code is::
-
-            0 = we're done, exit
-            1 = we've errored, exit with error string
-
+        :return: number of packages listed
         """
         if outputType in ['list', 'info', 'name', 'nevra']:
-            thingslisted = 0
             if len(lst) > 0:
-                thingslisted = 1
                 print('%s' % description)
                 info_set = set()
                 if outputType == 'list':
@@ -691,9 +637,7 @@ class Output(object):
                 if info_set:
                     print("\n".join(sorted(info_set)))
 
-            if thingslisted == 0:
-                return 1, [_('No packages to list')]
-            return 0, []
+            return len(lst)
 
     def userconfirm(self, msg=None, defaultyes_msg=None):
         """Get a yes or no from the user, and default to No
@@ -791,8 +735,8 @@ class Output(object):
         :param group: a Group object to output information about
         """
         def names(packages):
-            return sorted(pkg.name for pkg in packages)
-        print('\n' + _('Group: %s') % group.ui_name)
+            return sorted(dnf.util._name_unset_wrapper(pkg.name) for pkg in packages)
+        print(_('Group: %s') % dnf.util._name_unset_wrapper(group.ui_name))
 
         verbose = self.conf.verbose
         if verbose:
@@ -830,8 +774,8 @@ class Output(object):
         :param environment: an Environment object to output information about
         """
         def names(groups):
-            return sorted(group.name for group in groups)
-        print(_('Environment Group: %s') % environment.ui_name)
+            return sorted(dnf.util._name_unset_wrapper(group.name) for group in groups)
+        print(_('Environment Group: %s') % dnf.util._name_unset_wrapper(environment.ui_name))
 
         if self.conf.verbose:
             print(_(' Environment-Id: %s') % ucd(environment.id))
@@ -1056,37 +1000,6 @@ class Output(object):
             out[0:0] = self._banner(col_data, (_('Group'), _('Packages'), '', ''))
         return '\n'.join(out)
 
-    def _skipped_packages(self, report_problems, transaction):
-        """returns set of conflicting packages and set of packages with broken dependency that would
-        be additionally installed when --best and --allowerasing"""
-        if self.base._goal.actions & (hawkey.INSTALL | hawkey.UPGRADE | hawkey.UPGRADE_ALL):
-            best = True
-        else:
-            best = False
-        ng = deepcopy(self.base._goal)
-        params = {"allow_uninstall": self.base._allow_erasing,
-                  "force_best": best,
-                  "ignore_weak": True}
-        ret = ng.run(**params)
-        if not ret and report_problems:
-            msg = dnf.util._format_resolve_problems(ng.problem_rules())
-            logger.warning(msg)
-        problem_conflicts = set(ng.problem_conflicts(available=True))
-        problem_dependency = set(ng.problem_broken_dependency(available=True)) - problem_conflicts
-
-        def _nevra(item):
-            return hawkey.NEVRA(name=item.name, epoch=item.epoch, version=item.version,
-                                release=item.release, arch=item.arch)
-
-        # Sometimes, pkg is not in transaction item, therefore, comparing by nevra
-        transaction_nevras = [_nevra(tsi) for tsi in transaction]
-        skipped_conflicts = set(
-            [pkg for pkg in problem_conflicts if _nevra(pkg) not in transaction_nevras])
-        skipped_dependency = set(
-            [pkg for pkg in problem_dependency if _nevra(pkg) not in transaction_nevras])
-
-        return skipped_conflicts, skipped_dependency
-
     def list_transaction(self, transaction, total_width=None):
         """Return a string representation of the transaction in an
         easy-to-read format.
@@ -1101,7 +1014,7 @@ class Output(object):
             # in order to display module changes when RPM transaction is empty
             transaction = []
 
-        list_bunch = _make_lists(transaction, self.base._goal)
+        list_bunch = dnf.util._make_lists(transaction)
         pkglist_lines = []
         data = {'n' : {}, 'v' : {}, 'r' : {}}
         a_wid = 0 # Arch can't get "that big" ... so always use the max.
@@ -1222,8 +1135,7 @@ class Output(object):
             pkglist_lines.append((action, lines))
         if self.base._history:
             def format_line(group):
-                name = group.getName()
-                return (name if name else _("<name-unset>"), "", "", "", "", "", "")
+                return (dnf.util._name_unset_wrapper(group.getName()), "", "", "", "", "", "")
 
             install_env_group = self.base._history.env._installed
             if install_env_group:
@@ -1268,9 +1180,9 @@ class Output(object):
                     lines.append(format_line(group))
                 pkglist_lines.append((action, lines))
         # show skipped conflicting packages
-        if not self.conf.best and self.base._goal.actions & forward_actions:
+        if (not self.conf.best or not self.conf.strict) and self.base._goal.actions & forward_actions:
             lines = []
-            skipped_conflicts, skipped_broken = self._skipped_packages(
+            skipped_conflicts, skipped_broken = self.base._skipped_packages(
                 report_problems=True, transaction=transaction)
             skipped_broken = dict((str(pkg), pkg) for pkg in skipped_broken)
             for pkg in sorted(skipped_conflicts):
@@ -1435,13 +1347,8 @@ Transaction Summary
                                   max_msg_count, count, msg_pkgs))
         return ''.join(out)
 
-    def post_transaction_output(self, transaction):
-        """Returns a human-readable summary of the results of the
-        transaction.
 
-        :return: a string containing a human-readable summary of the
-           results of the transaction
-        """
+    def _pto_callback(self, action, tsis):
         #  Works a bit like calcColumns, but we never overflow a column we just
         # have a dynamic number of columns.
         def _fits_in_cols(msgs, num):
@@ -1471,61 +1378,33 @@ Transaction Summary
                 col_lens[col] *= -1
             return col_lens
 
-        def _tsi_or_pkg_nevra_cmp(item1, item2):
-            """Compares two transaction items or packages by nevra.
-               Used as a fallback when tsi does not contain package object.
-            """
-            ret = (item1.name > item2.name) - (item1.name < item2.name)
-            if ret != 0:
-                return ret
-            nevra1 = hawkey.NEVRA(name=item1.name, epoch=item1.epoch, version=item1.version,
-                                  release=item1.release, arch=item1.arch)
-            nevra2 = hawkey.NEVRA(name=item2.name, epoch=item2.epoch, version=item2.version,
-                                  release=item2.release, arch=item2.arch)
-            ret = nevra1.evr_cmp(nevra2, self.sack)
-            if ret != 0:
-                return ret
-            return (item1.arch > item2.arch) - (item1.arch < item2.arch)
-
-        out = ''
-        list_bunch = _make_lists(transaction, self.base._goal)
-
-        skipped_conflicts, skipped_broken = self._skipped_packages(
-            report_problems=False, transaction=transaction)
-        skipped = skipped_conflicts.union(skipped_broken)
-
-        for (action, tsis) in [(_('Upgraded'), list_bunch.upgraded),
-                               (_('Downgraded'), list_bunch.downgraded),
-                               (_('Installed'), list_bunch.installed +
-                                list_bunch.installed_group +
-                                list_bunch.installed_weak +
-                                list_bunch.installed_dep),
-                               (_('Reinstalled'), list_bunch.reinstalled),
-                               (_('Skipped'), skipped),
-                               (_('Removed'), list_bunch.erased +
-                                   list_bunch.erased_dep +
-                                   list_bunch.erased_clean),
-                               (_('Failed'), list_bunch.failed)]:
-            if not tsis:
-                continue
-            msgs = []
-            out += '\n%s:\n' % action
-            for tsi in sorted(tsis, key=functools.cmp_to_key(_tsi_or_pkg_nevra_cmp)):
-                msgs.append(str(tsi))
-            for num in (8, 7, 6, 5, 4, 3, 2):
-                cols = _fits_in_cols(msgs, num)
-                if cols:
-                    break
-            if not cols:
-                cols = [-(self.term.columns - 2)]
-            while msgs:
-                current_msgs = msgs[:len(cols)]
-                out += '  '
-                out += self.fmtColumns(zip(current_msgs, cols), end=u'\n')
-                msgs = msgs[len(cols):]
-
+        if not tsis:
+            return ''
+        out = []
+        msgs = []
+        out.append('{}:'.format(action))
+        for tsi in tsis:
+            msgs.append(str(tsi))
+        for num in (8, 7, 6, 5, 4, 3, 2):
+            cols = _fits_in_cols(msgs, num)
+            if cols:
+                break
+        if not cols:
+            cols = [-(self.term.columns - 2)]
+        while msgs:
+            current_msgs = msgs[:len(cols)]
+            out.append('  {}'.format(self.fmtColumns(zip(current_msgs, cols))))
+            msgs = msgs[len(cols):]
         return out
 
+
+    def post_transaction_output(self, transaction):
+        """
+        Return a human-readable summary of the transaction. Packages in sections
+        are arranged to columns.
+        """
+        return dnf.util._post_transaction_output(self.base, transaction, self._pto_callback)
+
     def setup_progress_callbacks(self):
         """Set up the progress callbacks and various
            output bars based on debug level.
@@ -1638,12 +1517,14 @@ Transaction Summary
             name = _("Command line")
             real_cols = self.term.real_columns
             if real_cols is None:
-                name_width = (
-                        24 if not transactions
-                        else max([len(t.cmdline) for t in transactions])
-                        )
-            else:
-                name_width = real_cols - 55 if real_cols > 79 else 24
+                # if output is redirected in `less` the columns
+                # detected are None value, to detect terminal size
+                # use stdin file descriptor
+                real_cols = dnf.cli.term._real_term_width(0)
+            if real_cols is None:
+                # if even stdin fd fails use 24 to fit to 80 cols
+                real_cols = 24
+            name_width = real_cols - 55 if real_cols > 79 else 24
         else:
             # TRANSLATORS: user names who executed transaction in history command output
             name = _("User name")
@@ -1667,7 +1548,12 @@ Transaction Summary
             else:
                 name = self._pwd_ui_username(transaction.loginuid, 24)
             name = ucd(name)
-            tm = time.strftime("%Y-%m-%d %H:%M",
+            # TRANSLATORS: This is the time format for dnf history list.
+            # You can change it but do it with caution because the output
+            # must be no longer than 16 characters. Format specifiers:
+            # %Y - year number (4 digits), %m - month (00-12), %d - day
+            # number (01-31), %H - hour (00-23), %M - minute (00-59).
+            tm = time.strftime(_("%Y-%m-%d %H:%M"),
                                time.localtime(transaction.beg_timestamp))
             num, uiacts = self._history_uiactions(transaction.data())
             name = fill_exact_width(name, name_width, name_width)
@@ -1726,7 +1612,7 @@ Transaction Summary
             if lastdbv is not None and trans.tid == lasttid:
                 #  If this is the last transaction, is good and it doesn't
                 # match the current rpmdb ... then mark it as bad.
-                rpmdbv = self.sack._rpmdb_version()
+                rpmdbv = self.base._ts.dbCookie()
                 trans.compare_rpmdbv(str(rpmdbv))
             lastdbv = None
 
@@ -1987,10 +1873,6 @@ Transaction Summary
 class DepSolveProgressCallBack(dnf.callback.Depsolve):
     """Provides text output callback functions for Dependency Solver callback."""
 
-    def __init__(self):
-        """requires yum-cli log and errorlog functions as arguments"""
-        self.loops = 0
-
     def pkg_added(self, pkg, mode):
         """Print information about a package being added to the
         transaction set.
@@ -2037,7 +1919,6 @@ class DepSolveProgressCallBack(dnf.callback.Depsolve):
         process.
         """
         logger.debug(_('--> Starting dependency resolution'))
-        self.loops += 1
 
     def end(self):
         """Output a message stating that dependency resolution has finished."""
@@ -2071,7 +1952,7 @@ class CliKeyImport(dnf.callback.KeyImport):
         return self.output.userconfirm()
 
 
-class CliTransactionDisplay(LoggingTransactionDisplay):
+class CliTransactionDisplay(TransactionDisplay):
     """A YUM specific callback class for RPM operations."""
 
     width = property(lambda self: dnf.cli.term._term_width())
@@ -2093,7 +1974,7 @@ class CliTransactionDisplay(LoggingTransactionDisplay):
         :param package: the package involved in the event
         :param action: the type of action that is taking place.  Valid
            values are given by
-           :func:`rpmtrans.LoggingTransactionDisplay.action.keys()`
+           :func:`rpmtrans.TransactionDisplay.action.keys()`
         :param ti_done: a number representing the amount of work
            already done in the current transaction
         :param ti_total: a number representing the total amount of work
@@ -2144,20 +2025,6 @@ class CliTransactionDisplay(LoggingTransactionDisplay):
                 if ti_done == ti_total:
                     print(" ")
 
-    def filelog(self, package, action):
-        pass
-
-    def error(self, message):
-        pass
-
-    def scriptout(self, msgs):
-        """Print messages originating from a package script.
-
-        :param msgs: the messages coming from the script
-        """
-        if msgs:
-            self.rpm_logger.info(ucd(msgs))
-
     def _makefmt(self, percent, ts_done, ts_total, progress=True,
                  pkgname=None, wid1=15):
         l = len(str(ts_total))
