@@ -148,7 +148,7 @@ class BaseConfig(object):
                     try:
                         self._config.optBinds().at(name).newString(priority, value)
                     except RuntimeError as e:
-                        logger.debug(_('Unknown configuration value: %s=%s in %s; %s'),
+                        logger.error(_('Invalid configuration value: %s=%s in %s; %s'),
                                      ucd(name), ucd(value), ucd(filename), str(e))
                 else:
                     if name == 'arch' and hasattr(self, name):
@@ -175,6 +175,24 @@ class BaseConfig(object):
 
         return '\n'.join(output) + '\n'
 
+    def set_or_append_opt_value(self, name, value_string, priority=PRIO_RUNTIME):
+        # :api
+        """For standard options, sets the value of the option if the priority is equal to or higher
+           than the current priority.
+           For "append" options, appends the values parsed from value_string to the current list of values. If the first
+           parsed element of the list of values is empty and the priority is equal to or higher than the current
+           priority, the current list is replaced with the new values.
+           If the priority is higher than the current priority, the current priority is increased to the priority.
+           Raises dnf.exceptions.ConfigError if the option with the given name does not exist or value_string contains
+           an invalid value or not allowed value.
+        """
+        opt_binds = self._config.optBinds()
+        try:
+            opt_binds.at(name).newString(priority, value_string)
+        except RuntimeError as e:
+            raise dnf.exceptions.ConfigError(
+                _('Cannot set "{}" to "{}": {}').format(name, value_string, str(e)), str(e))
+
     @staticmethod
     def write_raw_configfile(filename, section_id, substitutions, modify):
         # :api
@@ -233,8 +251,9 @@ class MainConf(BaseConfig):
         self.tempfiles = []
 
     def __del__(self):
-        for file_name in self.tempfiles:
-            os.unlink(file_name)
+        if hasattr(self, 'tempfiles'):
+            for file_name in self.tempfiles:
+                os.unlink(file_name)
 
     @property
     def get_reposdir(self):
@@ -269,7 +288,7 @@ class MainConf(BaseConfig):
                 temp_fd, temp_path = tempfile.mkstemp(prefix='dnf-downloaded-config-')
                 self.tempfiles.append(temp_path)
                 try:
-                    downloader.downloadURL(None, val, temp_fd)
+                    downloader.downloadURL(self._config, val, temp_fd)
                 except RuntimeError as e:
                     raise dnf.exceptions.ConfigError(
                         _('Configuration file URL "{}" could not be downloaded:\n'
@@ -324,7 +343,7 @@ class MainConf(BaseConfig):
                        'best', 'assumeyes', 'assumeno', 'clean_requirements_on_remove', 'gpgcheck',
                        'showdupesfromrepos', 'plugins', 'ip_resolve',
                        'rpmverbosity', 'disable_excludes', 'color',
-                       'downloadonly', 'exclude', 'excludepkgs', 'skip_broken',
+                       'downloadonly', 'persistence', 'exclude', 'excludepkgs', 'skip_broken',
                        'tsflags', 'arch', 'basearch', 'ignorearch', 'cacheonly', 'comment']
 
         for name in config_args:
@@ -406,11 +425,53 @@ class MainConf(BaseConfig):
     @releasever.setter
     def releasever(self, val):
         # :api
+        """
+        Sets the releasever variable and sets releasever_major and
+        releasever_minor accordingly. releasever_major is set to the part of
+        $releasever before the first ".". releasever_minor is set to the part
+        after the first ".".
+        """
         if val is None:
             self.substitutions.pop('releasever', None)
             return
         self.substitutions['releasever'] = str(val)
 
+    @property
+    def releasever_major(self):
+        # :api
+        return self.substitutions.get('releasever_major')
+
+    @releasever_major.setter
+    def releasever_major(self, val):
+        # :api
+        """
+        Override the releasever_major variable, which is usually derived from
+        the releasever variable. This setter does not update the value of
+        $releasever.
+        """
+        if val is None:
+            self.substitutions.pop('releasever_major', None)
+            return
+        self.substitutions['releasever_major'] = str(val)
+
+    @property
+    def releasever_minor(self):
+        # :api
+        return self.substitutions.get('releasever_minor')
+
+    @releasever_minor.setter
+    def releasever_minor(self, val):
+        # :api
+        """
+        Override the releasever_minor variable, which is usually derived from
+        the releasever variable. This setter does not update the value of
+        $releasever.
+        """
+        if val is None:
+            self.substitutions.pop('releasever_minor', None)
+            return
+        self.substitutions['releasever_minor'] = str(val)
+
     @property
     def arch(self):
         # :api
@@ -471,11 +532,11 @@ class RepoConf(BaseConfig):
     """Option definitions for repository INI file sections."""
 
     def __init__(self, parent, section=None, parser=None):
-        masterConfig = parent._config if parent else libdnf.conf.ConfigMain()
-        super(RepoConf, self).__init__(libdnf.conf.ConfigRepo(masterConfig), section, parser)
+        mainConfig = parent._config if parent else libdnf.conf.ConfigMain()
+        super(RepoConf, self).__init__(libdnf.conf.ConfigRepo(mainConfig), section, parser)
         # Do not remove! Attribute is a reference holder.
-        # Prevents premature removal of the masterConfig. The libdnf ConfigRepo points to it.
-        self._masterConfigRefHolder = masterConfig
+        # Prevents premature removal of the mainConfig. The libdnf ConfigRepo points to it.
+        self._mainConfigRefHolder = mainConfig
         if section:
             self._config.name().set(PRIO_DEFAULT, section)
 
